@@ -1,9 +1,12 @@
 import json
 import os
-from datetime import datetime
+import time
+from datetime import datetime, timedelta
+
 from typing import Dict, List, Any, Optional, Tuple, Union
 
 import pymysql
+import pymysqlpool
 from fuzzywuzzy import fuzz
 from pymysql import Connection
 from pytz import timezone
@@ -102,7 +105,7 @@ def find_best_match(item_name: str, items: List[Dict[str, Any]],
     :return: a tuple containing the best score and the best match
     """
     best_score, best_item = 0, None
-    common_words = {'arcane', 'prime', 'scene', 'set'}
+    common_words = {'prime', 'scene', 'set'}
 
     item_name = replace_aliases(item_name, word_aliases)
     item_name = remove_common_words(item_name, common_words)
@@ -196,6 +199,12 @@ class MarketDatabase:
     FROM item_subtypes 
     WHERE item_id=%s"""
 
+    _GET_AVERAGE_DEMAND_QUERY = """
+    SELECT item_id, average_demand
+    FROM item_average_demand
+    WHERE item_id IN ({}) AND platform = %s
+    """
+
     _GET_ITEM_STATISTICS_QUERY: str = """
     SELECT datetime, avg_price 
     FROM item_statistics 
@@ -251,19 +260,17 @@ class MarketDatabase:
     """
 
     _GET_PRICE_HISTORY_QUERY = """
-        SELECT datetime, avg_price
-        FROM item_statistics
-        WHERE item_id=%s
-        AND order_type='closed'
-        AND platform=%s
+    SELECT item_id, datetime, avg_price
+    FROM item_statistics
+    WHERE item_id IN ({}) AND platform=%s AND order_type='closed'
+    ORDER BY datetime
     """
 
     _GET_DEMAND_HISTORY_QUERY = """
-        SELECT datetime, volume
-        FROM item_statistics
-        WHERE item_id=%s
-        AND order_type='closed'
-        AND platform=%s
+    SELECT item_id, datetime, volume
+    FROM item_statistics
+    WHERE item_id IN ({}) AND platform=%s AND order_type='closed'
+    ORDER BY datetime
     """
 
     _GET_MOST_RECENT_DATE_QUERY = """
@@ -333,12 +340,12 @@ class MarketDatabase:
     WHERE id = %s"""
 
     _GET_LAST_AVERAGE_PRICES_QUERY = """
-    SELECT i.item_name, COALESCE(s.avg_price, 0) as last_average_price
+    SELECT i.item_name, COALESCE(s.median, 0) as last_average_price
     FROM items i
     LEFT JOIN (
-        SELECT item_id, avg_price
+        SELECT item_id, median
         FROM (
-            SELECT item_id, avg_price,
+            SELECT item_id, median,
                    ROW_NUMBER() OVER (PARTITION BY item_id ORDER BY datetime DESC) as rn
             FROM item_statistics
             WHERE platform = %s AND order_type = 'closed'
@@ -347,6 +354,30 @@ class MarketDatabase:
     ) s ON i.id = s.item_id
     """
 
+    _GET_ITEM_STATISTICS_DICT_QUERY = """
+    SELECT 
+        DATE(datetime) as date,
+        volume,
+        min_price,
+        max_price,
+        avg_price,
+        wa_price,
+        median,
+        moving_avg,
+        open_price,
+        closed_price,
+        mod_rank,
+        donch_bot,
+        donch_top
+    FROM item_statistics
+    WHERE item_id = %s
+    AND platform = %s
+    AND order_type = %s
+    AND datetime BETWEEN %s AND %s
+    ORDER BY datetime
+    """
+
+
     def __init__(self, user: str, password: str, host: str, database: str, initial_build: bool = False) -> None:
         """
         Initializes the database
@@ -355,10 +386,10 @@ class MarketDatabase:
         :param host: the host to connect to the database with
         :param database: the database to connect to
         """
-        self.connection: Connection = pymysql.connect(user=user,
-                                                      password=password,
-                                                      host=host,
-                                                      database=database)
+        config = {'host': host, 'user': user, 'password': password, 'database': database,
+                  'autocommit': True}
+
+        self.pool1 = pymysqlpool.ConnectionPool(pre_create_num=2, name='pool1', **config)
 
         self.users: Dict[str, str] = {}
 
@@ -396,9 +427,32 @@ class MarketDatabase:
         :param platform: the platform to fetch data for
         :return: a dictionary mapping item names to their last average price
         """
-        results = self.execute_query(self._GET_LAST_AVERAGE_PRICES_QUERY, platform, fetch='all')
-        return {item_name.lower(): float(price) for item_name, price in results}
+        cache_file = os.path.join(os.path.dirname(__file__), f"price_cache_{platform}.json")
 
+        # Check if we have a recent cache (less than 1 hour old)
+        if os.path.exists(cache_file):
+            cache_mtime = os.path.getmtime(cache_file)
+            if (time.time() - cache_mtime) < 3600:  # 1 hour in seconds
+                try:
+                    with open(cache_file, 'r') as f:
+                        logger.info(f"Loading item prices from cache for {platform}")
+                        return json.load(f)
+                except (json.JSONDecodeError, IOError):
+                    logger.warning("Failed to load price cache, rebuilding...")
+
+        # Cache doesn't exist or is too old, build it from database
+        logger.info(f"Building item price cache for {platform}")
+        results = self.execute_query(self._GET_LAST_AVERAGE_PRICES_QUERY, platform, fetch='all')
+        price_dict = {item_name.lower(): float(price) for item_name, price in results}
+
+        # Save the cache for next time
+        try:
+            with open(cache_file, 'w') as f:
+                json.dump(price_dict, f)
+        except IOError:
+            logger.warning("Failed to write price cache")
+
+        return price_dict
     def execute_query(self, query: str, *params, fetch: str = 'all',
                       commit: bool = False, many: bool = False) -> Union[Tuple, List[Tuple], None]:
         """
@@ -410,21 +464,20 @@ class MarketDatabase:
         :param many: whether or not to execute the query with multiple parameters
         :return: the result of the query if applicable
         """
-        self.connection.ping(reconnect=True)
+        with self.pool1.get_connection(pre_ping=True) as con1:
+            with con1.cursor() as cur:
+                if many:
+                    cur.executemany(query, params[0])
+                else:
+                    cur.execute(query, params)
 
-        with self.connection.cursor() as cur:
-            if many:
-                cur.executemany(query, params[0])
-            else:
-                cur.execute(query, params)
+                if commit:
+                    con1.commit()
 
-            if commit:
-                self.connection.commit()
-
-            if fetch == 'one':
-                return cur.fetchone()
-            elif fetch == 'all':
-                return cur.fetchall()
+                if fetch == 'one':
+                    return cur.fetchone()
+                elif fetch == 'all':
+                    return cur.fetchall()
 
     def get_all_items(self) -> List[dict]:
         """
@@ -592,13 +645,13 @@ class MarketDatabase:
 
         batch_size = 10_000
         total_batches = (len(values) + batch_size - 1) // batch_size
-
-        with self.connection.cursor() as cursor:
-            for i in range(0, len(values), batch_size):
-                batch_values = values[i:i + batch_size]
-                cursor.executemany(insert_query, batch_values)
-                logger.info(f"Progress: Batch {i // batch_size + 1} of {total_batches} completed")
-            self.connection.commit()
+        with self.pool1.get_connection(pre_ping=True) as con1:
+            with con1.cursor() as cursor:
+                for i in range(0, len(values), batch_size):
+                    batch_values = values[i:i + batch_size]
+                    cursor.executemany(insert_query, batch_values)
+                    logger.info(f"Progress: Batch {i // batch_size + 1} of {total_batches} completed")
+                self.connection.commit()
 
     def get_most_recent_statistic_date(self, platform: str = 'pc') -> Optional[datetime]:
         """
@@ -700,23 +753,81 @@ class MarketDatabase:
         """
         return self.execute_query(self._GET_ITEM_VOLUME_QUERY, days, item_id, platform, fetch='all')
 
-    def get_item_price_history(self, item_id: str, platform: str = 'pc') -> Dict[str, str]:
+    def get_item_price_history(self, item_ids: List[str], platform: str = 'pc') -> Dict[str, Dict[str, str]]:
         """
         Gets item price history from the database
         :param item_id: the item to get price history for
         :param platform: the platform to fetch data for
         :return: the item price history
         """
-        return dict(self.execute_query(self._GET_PRICE_HISTORY_QUERY, item_id, platform, fetch='all'))
+        placeholders = ','.join(['%s'] * len(item_ids))
+        query = self._GET_PRICE_HISTORY_QUERY.format(placeholders)
+        results = self.execute_query(query, *item_ids, platform, fetch='all')
 
-    def get_item_demand_history(self, item_id: str, platform: str = 'pc') -> Dict[str, str]:
+        price_history = {}
+        for item_id, datetime, avg_price in results:
+            if item_id not in price_history:
+                price_history[item_id] = {}
+            price_history[item_id][datetime] = avg_price
+
+        return price_history
+
+    def update_average_demand(self, platform: str = 'pc'):
+        """
+        Updates the average demand for items in the database
+        :param platform: the platform to update data for
+        :return: None
+        """
+        update_query = """
+        INSERT INTO item_average_demand (item_id, platform, average_demand, last_updated)
+        SELECT 
+            item_id, 
+            %s AS platform,
+            AVG(volume) AS average_demand,
+            NOW() AS last_updated
+        FROM item_statistics
+        WHERE platform = %s
+          AND order_type = 'closed'
+          AND datetime >= DATE_SUB(NOW(), INTERVAL 90 DAY)
+        GROUP BY item_id
+        ON DUPLICATE KEY UPDATE
+            average_demand = VALUES(average_demand),
+            last_updated = VALUES(last_updated)
+        """
+        self.execute_query(update_query, platform, platform)
+
+    def get_item_demand_history(self, item_ids: List[str], platform: str = 'pc') -> Dict[str, Dict[str, str]]:
         """
         Gets item demand history from the database
         :param item_id: the item to get demand history for
         :param platform: the platform to fetch data for
         :return: the item demand history
         """
-        return dict(self.execute_query(self._GET_DEMAND_HISTORY_QUERY, item_id, platform, fetch='all'))
+        placeholders = ','.join(['%s'] * len(item_ids))
+        query = self._GET_DEMAND_HISTORY_QUERY.format(placeholders)
+        results = self.execute_query(query, *item_ids, platform, fetch='all')
+
+        demand_history = {}
+        for item_id, datetime, volume in results:
+            if item_id not in demand_history:
+                demand_history[item_id] = {}
+            demand_history[item_id][datetime] = volume
+
+        return demand_history
+
+    def get_item_average_demand(self, item_ids: List[str], platform: str = 'pc') -> Dict[str, float]:
+        """
+        Gets pre-calculated average demand for items from the database
+        :param item_ids: the items to get average demand for
+        :param platform: the platform to fetch data for
+        :return: a dictionary of item_id to average demand
+        """
+        placeholders = ','.join(['%s'] * len(item_ids))
+        query = self._GET_AVERAGE_DEMAND_QUERY.format(placeholders)
+        results = self.execute_query(query, *item_ids, platform, fetch='all')
+
+        average_demand = {item_id: avg_demand for item_id, avg_demand in results}
+        return average_demand
 
     def _get_fuzzy_item(self, item_name: str) -> Optional[Dict[str, str]]:
         """
@@ -804,6 +915,69 @@ class MarketDatabase:
         if history_queries:
             self.execute_query(self._INSERT_USERNAME_HISTORY_QUERY, history_queries, commit=True, many=True)
 
+    def get_item_statistics_dict(self, item_id: str, platform: str = 'pc', order_type: str = 'closed',
+                                 start_date: Optional[datetime] = None, end_date: Optional[datetime] = None,
+                                 days: Optional[int] = None, fields: Optional[List[str]] = None) -> Dict[
+        str, Dict[str, Any]]:
+        """
+        Retrieves filtered statistic data for a given item in a dictionary format.
+
+        :param item_id: The ID of the item to retrieve statistics for.
+        :param platform: The platform to fetch data for (default is 'pc').
+        :param order_type: The type of order to filter by (default is 'closed').
+        :param start_date: The start date for filtering data.
+        :param end_date: The end date for filtering data (default is the most recent date with data).
+        :param days: Number of days to retrieve data for (overrides start_date if provided).
+        :param fields: List of fields to include in the result (default is all fields).
+        :return: A dictionary where keys are dates and values are dictionaries of statistic data.
+        """
+        most_recent_date = self.get_most_recent_statistic_date(platform)
+
+        if most_recent_date is None:
+            return {}
+
+        if end_date is None or end_date > most_recent_date:
+            end_date = most_recent_date
+
+        if days is not None:
+            start_date = end_date - timedelta(days=days - 1)  # -1 because we want to include the end_date
+        elif start_date is None:
+            start_date = end_date - timedelta(days=364)  # Default to last 365 days
+
+        results = self.execute_query(self._GET_ITEM_STATISTICS_DICT_QUERY,
+                                     item_id, platform, order_type, start_date, end_date,
+                                     fetch='all')
+
+        return self._process_statistics_results(results, fields)
+
+    @staticmethod
+    def _process_statistics_results(results: List[Tuple], fields: Optional[List[str]] = None) -> Dict[
+        str, Dict[str, Any]]:
+        """
+        Helper function to process and filter statistics results.
+
+        :param results: Raw results from the database query.
+        :param fields: List of fields to include in the result (default is all fields).
+        :return: Processed and filtered statistics dictionary.
+        """
+        all_fields = ['volume', 'min_price', 'max_price', 'avg_price', 'wa_price', 'median',
+                      'moving_avg', 'open_price', 'closed_price', 'mod_rank', 'donch_bot', 'donch_top']
+
+        if fields is None:
+            fields = all_fields
+        else:
+            fields = [field for field in fields if field in all_fields]
+
+        statistics_dict = {}
+        for row in results:
+            date = row[0].strftime('%Y-%m-%d')
+            statistics_dict[date] = {
+                field: float(row[all_fields.index(field) + 1]) if row[all_fields.index(field) + 1] is not None else None
+                for field in fields
+            }
+
+        return statistics_dict
+
     def get_price_history_dicts(self, item_names: List[str], platform: str = 'pc') -> Dict[str, Dict[str, str]]:
         """
         Gets price history dictionaries for a list of item names
